@@ -2,7 +2,7 @@
 // detail/socket_ops.hpp
 // ~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2015 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2020 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -18,9 +18,8 @@
 #include "asio/detail/config.hpp"
 
 #include "asio/error_code.hpp"
-#include "asio/detail/shared_ptr.hpp"
+#include "asio/detail/memory.hpp"
 #include "asio/detail/socket_types.hpp"
-#include "asio/detail/weak_ptr.hpp"
 
 #include "asio/detail/push_options.hpp"
 
@@ -140,9 +139,15 @@ ASIO_DECL void init_buf(buf& b, const void* data, size_t size);
 ASIO_DECL signed_size_type recv(socket_type s, buf* bufs,
     size_t count, int flags, asio::error_code& ec);
 
+ASIO_DECL signed_size_type recv1(socket_type s,
+    void* data, size_t size, int flags, asio::error_code& ec);
+
 ASIO_DECL size_t sync_recv(socket_type s, state_type state, buf* bufs,
     size_t count, int flags, bool all_empty, asio::error_code& ec);
 
+ASIO_DECL size_t sync_recv1(socket_type s, state_type state,
+    void* data, size_t size, int flags, asio::error_code& ec);
+
 #if defined(ASIO_HAS_IOCP)
 
 ASIO_DECL void complete_iocp_recv(state_type state,
@@ -155,16 +160,28 @@ ASIO_DECL bool non_blocking_recv(socket_type s,
     buf* bufs, size_t count, int flags, bool is_stream,
     asio::error_code& ec, size_t& bytes_transferred);
 
+ASIO_DECL bool non_blocking_recv1(socket_type s,
+    void* data, size_t size, int flags, bool is_stream,
+    asio::error_code& ec, size_t& bytes_transferred);
+
 #endif // defined(ASIO_HAS_IOCP)
 
 ASIO_DECL signed_size_type recvfrom(socket_type s, buf* bufs,
     size_t count, int flags, socket_addr_type* addr,
     std::size_t* addrlen, asio::error_code& ec);
 
+ASIO_DECL signed_size_type recvfrom1(socket_type s, void* data,
+    size_t size, int flags, socket_addr_type* addr,
+    std::size_t* addrlen, asio::error_code& ec);
+
 ASIO_DECL size_t sync_recvfrom(socket_type s, state_type state,
     buf* bufs, size_t count, int flags, socket_addr_type* addr,
     std::size_t* addrlen, asio::error_code& ec);
 
+ASIO_DECL size_t sync_recvfrom1(socket_type s, state_type state,
+    void* data, size_t size, int flags, socket_addr_type* addr,
+    std::size_t* addrlen, asio::error_code& ec);
+
 #if defined(ASIO_HAS_IOCP)
 
 ASIO_DECL void complete_iocp_recvfrom(
@@ -178,6 +195,11 @@ ASIO_DECL bool non_blocking_recvfrom(socket_type s,
     socket_addr_type* addr, std::size_t* addrlen,
     asio::error_code& ec, size_t& bytes_transferred);
 
+ASIO_DECL bool non_blocking_recvfrom1(socket_type s,
+    void* data, size_t size, int flags,
+    socket_addr_type* addr, std::size_t* addrlen,
+    asio::error_code& ec, size_t& bytes_transferred);
+
 #endif // defined(ASIO_HAS_IOCP)
 
 ASIO_DECL signed_size_type recvmsg(socket_type s, buf* bufs,
@@ -205,10 +227,16 @@ ASIO_DECL bool non_blocking_recvmsg(socket_type s,
 ASIO_DECL signed_size_type send(socket_type s, const buf* bufs,
     size_t count, int flags, asio::error_code& ec);
 
+ASIO_DECL signed_size_type send1(socket_type s,
+    const void* data, size_t size, int flags, asio::error_code& ec);
+
 ASIO_DECL size_t sync_send(socket_type s, state_type state,
     const buf* bufs, size_t count, int flags,
     bool all_empty, asio::error_code& ec);
 
+ASIO_DECL size_t sync_send1(socket_type s, state_type state,
+    const void* data, size_t size, int flags, asio::error_code& ec);
+
 #if defined(ASIO_HAS_IOCP)
 
 ASIO_DECL void complete_iocp_send(
@@ -221,16 +249,28 @@ ASIO_DECL bool non_blocking_send(socket_type s,
     const buf* bufs, size_t count, int flags,
     asio::error_code& ec, size_t& bytes_transferred);
 
+ASIO_DECL bool non_blocking_send1(socket_type s,
+    const void* data, size_t size, int flags,
+    asio::error_code& ec, size_t& bytes_transferred);
+
 #endif // defined(ASIO_HAS_IOCP)
 
 ASIO_DECL signed_size_type sendto(socket_type s, const buf* bufs,
     size_t count, int flags, const socket_addr_type* addr,
     std::size_t addrlen, asio::error_code& ec);
 
+ASIO_DECL signed_size_type sendto1(socket_type s, const void* data,
+    size_t size, int flags, const socket_addr_type* addr,
+    std::size_t addrlen, asio::error_code& ec);
+
 ASIO_DECL size_t sync_sendto(socket_type s, state_type state,
     const buf* bufs, size_t count, int flags, const socket_addr_type* addr,
     std::size_t addrlen, asio::error_code& ec);
 
+ASIO_DECL size_t sync_sendto1(socket_type s, state_type state,
+    const void* data, size_t size, int flags, const socket_addr_type* addr,
+    std::size_t addrlen, asio::error_code& ec);
+
 #if !defined(ASIO_HAS_IOCP)
 
 ASIO_DECL bool non_blocking_sendto(socket_type s,
@@ -238,6 +278,11 @@ ASIO_DECL bool non_blocking_sendto(socket_type s,
     const socket_addr_type* addr, std::size_t addrlen,
     asio::error_code& ec, size_t& bytes_transferred);
 
+ASIO_DECL bool non_blocking_sendto1(socket_type s,
+    const void* data, size_t size, int flags,
+    const socket_addr_type* addr, std::size_t addrlen,
+    asio::error_code& ec, size_t& bytes_transferred);
+
 #endif // !defined(ASIO_HAS_IOCP)
 
 ASIO_DECL socket_type socket(int af, int type, int protocol,
@@ -264,12 +309,16 @@ ASIO_DECL int select(int nfds, fd_set* readfds, fd_set* writefds,
     fd_set* exceptfds, timeval* timeout, asio::error_code& ec);
 
 ASIO_DECL int poll_read(socket_type s,
-    state_type state, asio::error_code& ec);
+    state_type state, int msec, asio::error_code& ec);
 
 ASIO_DECL int poll_write(socket_type s,
-    state_type state, asio::error_code& ec);
+    state_type state, int msec, asio::error_code& ec);
 
-ASIO_DECL int poll_connect(socket_type s, asio::error_code& ec);
+ASIO_DECL int poll_error(socket_type s,
+    state_type state, int msec, asio::error_code& ec);
+
+ASIO_DECL int poll_connect(socket_type s,
+    int msec, asio::error_code& ec);
 
 #endif // !defined(ASIO_WINDOWS_RUNTIME)
 
