@@ -2,14 +2,14 @@
 // impl/src.cpp
 // ~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2015 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2020 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
 
 #if defined(_MSC_VER) \
-  || defined(__BORLANDC__) \
+  || (defined(__BORLANDC__) && !defined(__clang__)) \
   || defined(__DMC__)
 # pragma message ( \
     "This file is deprecated. " \
