@@ -2,7 +2,7 @@
 // blocking_udp_echo_server.cpp
 // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2020 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -16,9 +16,9 @@ using asio::ip::udp;
 
 enum { max_length = 1024 };
 
-void server(asio::io_service& io_service, unsigned short port)
+void server(asio::io_context& io_context, unsigned short port)
 {
-  udp::socket sock(io_service, udp::endpoint(udp::v4(), port));
+  udp::socket sock(io_context, udp::endpoint(udp::v4(), port));
   for (;;)
   {
     char data[max_length];
@@ -39,10 +39,10 @@ int main(int argc, char* argv[])
       return 1;
     }
 
-    asio::io_service io_service;
+    asio::io_context io_context;
 
     using namespace std; // For atoi.
-    server(io_service, atoi(argv[1]));
+    server(io_context, atoi(argv[1]));
   }
   catch (std::exception& e)
   {
