@@ -2,7 +2,7 @@
 // server.hpp
 // ~~~~~~~~~~
 //
-// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2020 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -31,7 +31,7 @@ public:
   explicit server(const std::string& address, const std::string& port,
       const std::string& doc_root);
 
-  /// Run the server's io_service loop.
+  /// Run the server's io_context loop.
   void run();
 
 private:
@@ -44,8 +44,8 @@ private:
   /// Handle a request to stop the server.
   void handle_stop();
 
-  /// The io_service used to perform asynchronous operations.
-  asio::io_service io_service_;
+  /// The io_context used to perform asynchronous operations.
+  asio::io_context io_context_;
 
   /// The signal_set is used to register for process termination notifications.
   asio::signal_set signals_;
