@@ -1,8 +1,8 @@
 //
-// sync_client.cpp
+// http_client.cpp
 // ~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2020 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -33,7 +33,7 @@ int main(int argc, char* argv[])
     // The entire sequence of I/O operations must complete within 60 seconds.
     // If an expiry occurs, the socket is automatically closed and the stream
     // becomes bad.
-    s.expires_from_now(boost::posix_time::seconds(60));
+    s.expires_after(asio::chrono::seconds(60));
 
     // Establish a connection to the server.
     s.connect(argv[1], "http");
