@@ -2,7 +2,7 @@
 // connect_pair.cpp
 // ~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2020 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -13,7 +13,7 @@
 #include <cctype>
 #include <asio.hpp>
 #include <boost/array.hpp>
-#include <boost/bind.hpp>
+#include <boost/bind/bind.hpp>
 
 #if defined(ASIO_HAS_LOCAL_SOCKETS)
 
@@ -22,8 +22,8 @@ using asio::local::stream_protocol;
 class uppercase_filter
 {
 public:
-  uppercase_filter(asio::io_service& io_service)
-    : socket_(io_service)
+  uppercase_filter(asio::io_context& io_context)
+    : socket_(io_context)
   {
   }
 
@@ -81,11 +81,11 @@ private:
   boost::array<char, 512> data_;
 };
 
-void run(asio::io_service* io_service)
+void run(asio::io_context* io_context)
 {
   try
   {
-    io_service->run();
+    io_context->run();
   }
   catch (std::exception& e)
   {
@@ -98,16 +98,16 @@ int main()
 {
   try
   {
-    asio::io_service io_service;
+    asio::io_context io_context;
 
     // Create filter and establish a connection to it.
-    uppercase_filter filter(io_service);
-    stream_protocol::socket socket(io_service);
+    uppercase_filter filter(io_context);
+    stream_protocol::socket socket(io_context);
     asio::local::connect_pair(socket, filter.socket());
     filter.start();
 
-    // The io_service runs in a background thread to perform filtering.
-    asio::thread thread(boost::bind(run, &io_service));
+    // The io_context runs in a background thread to perform filtering.
+    asio::thread thread(boost::bind(run, &io_context));
 
     for (;;)
     {
