@@ -2,7 +2,7 @@
 // sync_client.cpp
 // ~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2020 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -31,22 +31,21 @@ int main(int argc, char* argv[])
       return 1;
     }
 
-    asio::io_service io_service;
+    asio::io_context io_context;
 
     // Get a list of endpoints corresponding to the SOCKS 4 server name.
-    tcp::resolver resolver(io_service);
-    tcp::resolver::query socks_query(argv[1], argv[2]);
-    tcp::resolver::iterator endpoint_iterator = resolver.resolve(socks_query);
+    tcp::resolver resolver(io_context);
+    tcp::resolver::results_type endpoints = resolver.resolve(argv[1], argv[2]);
 
     // Try each endpoint until we successfully establish a connection to the
     // SOCKS 4 server.
-    tcp::socket socket(io_service);
-    asio::connect(socket, endpoint_iterator);
+    tcp::socket socket(io_context);
+    asio::connect(socket, endpoints);
 
     // Get an endpoint for the Boost website. This will be passed to the SOCKS
     // 4 server. Explicitly specify IPv4 since SOCKS 4 does not support IPv6.
-    tcp::resolver::query http_query(tcp::v4(), "www.boost.org", "http");
-    tcp::endpoint http_endpoint = *resolver.resolve(http_query);
+    tcp::endpoint http_endpoint =
+      *resolver.resolve(tcp::v4(), "www.boost.org", "http").begin();
 
     // Send the request to the SOCKS 4 server.
     socks4::request socks_request(
