@@ -2,7 +2,7 @@
 // timer.cpp
 // ~~~~~~~~~
 //
-// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2020 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -10,7 +10,6 @@
 
 #include <iostream>
 #include <asio.hpp>
-#include <boost/date_time/posix_time/posix_time.hpp>
 
 void print(const asio::error_code& /*e*/)
 {
@@ -19,9 +18,9 @@ void print(const asio::error_code& /*e*/)
 
 int main()
 {
-  asio::io_service io;
+  asio::io_context io;
 
-  asio::deadline_timer t(io, boost::posix_time::seconds(5));
+  asio::steady_timer t(io, asio::chrono::seconds(5));
   t.async_wait(&print);
 
   io.run();
