@@ -2,7 +2,7 @@
 // server.cpp
 // ~~~~~~~~~~
 //
-// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2020 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -17,11 +17,10 @@ namespace server {
 
 server::server(const std::string& address, const std::string& port,
     const std::string& doc_root)
-  : io_service_(),
-    signals_(io_service_),
-    acceptor_(io_service_),
+  : io_context_(1),
+    signals_(io_context_),
+    acceptor_(io_context_),
     connection_manager_(),
-    socket_(io_service_),
     request_handler_(doc_root)
 {
   // Register to handle the signals that indicate when the server should exit.
@@ -36,8 +35,9 @@ server::server(const std::string& address, const std::string& port,
   do_await_stop();
 
   // Open the acceptor with the option to reuse the address (i.e. SO_REUSEADDR).
-  asio::ip::tcp::resolver resolver(io_service_);
-  asio::ip::tcp::endpoint endpoint = *resolver.resolve({address, port});
+  asio::ip::tcp::resolver resolver(io_context_);
+  asio::ip::tcp::endpoint endpoint =
+    *resolver.resolve(address, port).begin();
   acceptor_.open(endpoint.protocol());
   acceptor_.set_option(asio::ip::tcp::acceptor::reuse_address(true));
   acceptor_.bind(endpoint);
@@ -48,17 +48,17 @@ server::server(const std::string& address, const std::string& port,
 
 void server::run()
 {
-  // The io_service::run() call will block until all asynchronous operations
+  // The io_context::run() call will block until all asynchronous operations
   // have finished. While the server is running, there is always at least one
   // asynchronous operation outstanding: the asynchronous accept call waiting
   // for new incoming connections.
-  io_service_.run();
+  io_context_.run();
 }
 
 void server::do_accept()
 {
-  acceptor_.async_accept(socket_,
-      [this](std::error_code ec)
+  acceptor_.async_accept(
+      [this](std::error_code ec, asio::ip::tcp::socket socket)
       {
         // Check whether the server was stopped by a signal before this
         // completion handler had a chance to run.
@@ -70,7 +70,7 @@ void server::do_accept()
         if (!ec)
         {
           connection_manager_.start(std::make_shared<connection>(
-              std::move(socket_), connection_manager_, request_handler_));
+              std::move(socket), connection_manager_, request_handler_));
         }
 
         do_accept();
@@ -83,7 +83,7 @@ void server::do_await_stop()
       [this](std::error_code /*ec*/, int /*signo*/)
       {
         // The server is stopped by cancelling all outstanding asynchronous
-        // operations. Once all operations have finished the io_service::run()
+        // operations. Once all operations have finished the io_context::run()
         // call will exit.
         acceptor_.close();
         connection_manager_.stop_all();
