@@ -2,7 +2,7 @@
 // stream.cpp
 // ~~~~~~~~~~
 //
-// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2020 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -30,12 +30,10 @@
 
 namespace ssl_stream_compile {
 
-#if !defined(ASIO_ENABLE_OLD_SSL)
 bool verify_callback(bool, asio::ssl::verify_context&)
 {
   return false;
 }
-#endif // !defined(ASIO_ENABLE_OLD_SSL)
 
 void handshake_handler(const asio::error_code&)
 {
@@ -64,44 +62,44 @@ void test()
 
   try
   {
-    io_service ios;
+    io_context ioc;
     char mutable_char_buffer[128] = "";
     const char const_char_buffer[128] = "";
-    asio::ssl::context context(ios, asio::ssl::context::sslv23);
+    asio::ssl::context context(asio::ssl::context::sslv23);
     archetypes::lazy_handler lazy;
     asio::error_code ec;
 
     // ssl::stream constructors.
 
-    ssl::stream<ip::tcp::socket> stream1(ios, context);
-    ip::tcp::socket socket1(ios, ip::tcp::v4());
+    ssl::stream<ip::tcp::socket> stream1(ioc, context);
+    ip::tcp::socket socket1(ioc, ip::tcp::v4());
     ssl::stream<ip::tcp::socket&> stream2(socket1, context);
 
+#if defined(ASIO_HAS_MOVE)
+    ssl::stream<ip::tcp::socket> stream3
+      = ssl::stream<ip::tcp::socket>(ioc, context);
+    ssl::stream<ip::tcp::socket> stream4(std::move(stream3));
+#endif // defined(ASIO_HAS_MOVE)
+
     // basic_io_object functions.
 
-    io_service& ios_ref = stream1.get_io_service();
-    (void)ios_ref;
+    ssl::stream<ip::tcp::socket>::executor_type ex = stream1.get_executor();
+    (void)ex;
 
     // ssl::stream functions.
 
-#if !defined(ASIO_ENABLE_OLD_SSL)
     SSL* ssl1 = stream1.native_handle();
     (void)ssl1;
-#endif // !defined(ASIO_ENABLE_OLD_SSL)
-
-    SSL* ssl2 = stream1.impl()->ssl;
-    (void)ssl2;
 
     ssl::stream<ip::tcp::socket>::lowest_layer_type& lowest_layer
       = stream1.lowest_layer();
     (void)lowest_layer;
 
-    const ssl::stream<ip::tcp::socket>& stream3 = stream1;
+    const ssl::stream<ip::tcp::socket>& stream5 = stream1;
     const ssl::stream<ip::tcp::socket>::lowest_layer_type& lowest_layer2
-      = stream3.lowest_layer();
+      = stream5.lowest_layer();
     (void)lowest_layer2;
 
-#if !defined(ASIO_ENABLE_OLD_SSL)
     stream1.set_verify_mode(ssl::verify_none);
     stream1.set_verify_mode(ssl::verify_none, ec);
 
@@ -110,14 +108,12 @@ void test()
 
     stream1.set_verify_callback(verify_callback);
     stream1.set_verify_callback(verify_callback, ec);
-#endif // !defined(ASIO_ENABLE_OLD_SSL)
 
     stream1.handshake(ssl::stream_base::client);
     stream1.handshake(ssl::stream_base::server);
     stream1.handshake(ssl::stream_base::client, ec);
     stream1.handshake(ssl::stream_base::server, ec);
 
-#if !defined(ASIO_ENABLE_OLD_SSL)
     stream1.handshake(ssl::stream_base::client, buffer(mutable_char_buffer));
     stream1.handshake(ssl::stream_base::server, buffer(mutable_char_buffer));
     stream1.handshake(ssl::stream_base::client, buffer(const_char_buffer));
@@ -130,7 +126,6 @@ void test()
         buffer(const_char_buffer), ec);
     stream1.handshake(ssl::stream_base::server,
         buffer(const_char_buffer), ec);
-#endif // !defined(ASIO_ENABLE_OLD_SSL)
 
     stream1.async_handshake(ssl::stream_base::client, handshake_handler);
     stream1.async_handshake(ssl::stream_base::server, handshake_handler);
@@ -139,7 +134,6 @@ void test()
     int i2 = stream1.async_handshake(ssl::stream_base::server, lazy);
     (void)i2;
 
-#if !defined(ASIO_ENABLE_OLD_SSL)
     stream1.async_handshake(ssl::stream_base::client,
         buffer(mutable_char_buffer), buffered_handshake_handler);
     stream1.async_handshake(ssl::stream_base::server,
@@ -160,7 +154,6 @@ void test()
     int i6 = stream1.async_handshake(ssl::stream_base::server,
         buffer(const_char_buffer), lazy);
     (void)i6;
-#endif // !defined(ASIO_ENABLE_OLD_SSL)
 
     stream1.shutdown();
     stream1.shutdown(ec);
@@ -187,16 +180,6 @@ void test()
     stream1.async_read_some(buffer(mutable_char_buffer), read_some_handler);
     int i10 = stream1.async_read_some(buffer(mutable_char_buffer), lazy);
     (void)i10;
-
-#if defined(ASIO_ENABLE_OLD_SSL)
-    stream1.peek(buffer(mutable_char_buffer));
-    stream1.peek(buffer(mutable_char_buffer), ec);
-
-    std::size_t in_avail1 = stream1.in_avail();
-    (void)in_avail1;
-    std::size_t in_avail2 = stream1.in_avail(ec);
-    (void)in_avail2;
-#endif // defined(ASIO_ENABLE_OLD_SSL)
   }
   catch (std::exception&)
   {
