@@ -2,7 +2,7 @@
 // basic_random_access_handle.cpp
 // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2020 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -16,11 +16,10 @@
 // Test that header file is self-contained.
 #include "asio/windows/basic_random_access_handle.hpp"
 
-#include "asio.hpp"
 #include "../unit_test.hpp"
 
 ASIO_TEST_SUITE
 (
-  "windows/basic_random_access_handle",
+  "basic_random_access_handle",
   ASIO_TEST_CASE(null_test)
 )
